"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RepeatField = RepeatField;

function _css() {
  const data = require("@emotion/css");

  _css = function _css() {
    return data;
  };

  return data;
}

function _antd() {
  const data = require("antd");

  _antd = function _antd() {
    return data;
  };

  return data;
}

function _react() {
  const data = _interopRequireDefault(require("react"));

  _react = function _react() {
    return data;
  };

  return data;
}

function _reactI18next() {
  const data = require("react-i18next");

  _reactI18next = function _reactI18next() {
    return data;
  };

  return data;
}

function _reactJsCron() {
  const data = require("react-js-cron");

  _reactJsCron = function _reactJsCron() {
    return data;
  };

  return data;
}

var _Cron = _interopRequireDefault(require("./locale/Cron.zh-CN"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const languages = {
  'zh-CN': _Cron.default
};
const RepeatOptions = [{
  value: 'none',
  text: 'No repeat'
}, {
  value: 60000,
  text: 'By minute',
  unitText: 'Minutes'
}, {
  value: 3600000,
  text: 'By hour',
  unitText: 'Hours'
}, {
  value: 86400000,
  text: 'By day',
  unitText: 'Days'
}, {
  value: 604800000,
  text: 'By week',
  unitText: 'Weeks'
}, // { value: 18144_000_000, text: 'By 30 days' },
{
  value: 'cron',
  text: 'Advanced'
}];

function getNumberOption(v) {
  const opts = RepeatOptions.filter(option => typeof option.value === 'number').reverse();
  return opts.find(item => !(v % item.value));
}

function getRepeatTypeValue(v) {
  switch (typeof v) {
    case 'number':
      const option = getNumberOption(v);
      return option ? option.value : 'none';

    case 'string':
      return 'cron';

    default:
      break;
  }

  return 'none';
}

function CommonRepeatField({
  value,
  onChange: _onChange
}) {
  const _useTranslation = (0, _reactI18next().useTranslation)(),
        t = _useTranslation.t;

  const option = getNumberOption(value);
  return _react().default.createElement(_antd().InputNumber, {
    value: value / option.value,
    onChange: v => _onChange(v * option.value),
    min: 1,
    addonBefore: t('Every'),
    addonAfter: t(option.unitText)
  });
}

function RepeatField({
  value = null,
  onChange
}) {
  const _useTranslation2 = (0, _reactI18next().useTranslation)(),
        t = _useTranslation2.t;

  const typeValue = getRepeatTypeValue(value);

  function onTypeChange(v) {
    if (v === 'none') {
      onChange(null);
      return;
    }

    if (v === 'cron') {
      onChange('0 * * * * *');
      return;
    }

    onChange(v);
  }

  const locale = languages[localStorage.getItem('NOCOBASE_LOCALE') || 'en-US'];
  return _react().default.createElement("fieldset", {
    className: (0, _css().css)`
      display: flex;
      flex-direction: ${typeValue === 'cron' ? 'column' : 'row'};
      align-items: flex-start;
      gap: .5em;

      .react-js-cron{
        padding: .5em .5em 0 .5em;
        border: 1px dashed #ccc;

        .react-js-cron-field{
          margin-bottom: .5em;

          > span{
            margin: 0 .5em 0 0;
          }

          > .react-js-cron-select{
            margin: 0 .5em 0 0;
          }
        }
      }
    `
  }, _react().default.createElement(_antd().Select, {
    value: typeValue,
    onChange: onTypeChange
  }, RepeatOptions.map(item => _react().default.createElement(_antd().Select.Option, {
    key: item.value,
    value: item.value
  }, t(item.text)))), typeof typeValue === 'number' ? _react().default.createElement(CommonRepeatField, {
    value: value,
    onChange: onChange
  }) : null, typeValue === 'cron' ? _react().default.createElement(_reactJsCron().Cron, {
    value: value.trim().split(/\s+/).slice(1).join(' '),
    setValue: v => onChange(`0 ${v}`),
    clearButton: false,
    locale: locale
  }) : null);
}