import { Transaction, Transactionable } from 'sequelize';
import Plugin from '.';
import ExecutionModel from './models/Execution';
import JobModel from './models/Job';
import FlowNodeModel from './models/FlowNode';
export interface ProcessorOptions extends Transactionable {
    _context?: any;
    plugin: Plugin;
}
export default class Processor {
    execution: ExecutionModel;
    options: ProcessorOptions;
    static StatusMap: {
        [x: number]: number;
    };
    transaction: Transaction;
    nodes: FlowNodeModel[];
    nodesMap: Map<number, FlowNodeModel>;
    jobsMap: Map<number, JobModel>;
    jobsMapByNodeId: {
        [key: number]: any;
    };
    constructor(execution: ExecutionModel, options: ProcessorOptions);
    private makeNodes;
    private makeJobs;
    private getTransaction;
    prepare(commit?: boolean): Promise<void>;
    start(): Promise<void>;
    resume(job: JobModel): Promise<void>;
    private commit;
    private exec;
    run(node: any, input?: any): any;
    end(node: any, job: any): Promise<any>;
    recall(node: any, job: any): Promise<any>;
    exit(job: JobModel | null): Promise<any>;
    saveJob(payload: any): Promise<any>;
    getBranches(node: FlowNodeModel): FlowNodeModel[];
    findBranchStartNode(node: FlowNodeModel, parent?: FlowNodeModel): FlowNodeModel | null;
    findBranchParentNode(node: FlowNodeModel): FlowNodeModel | null;
    findBranchParentJob(job: JobModel, node: FlowNodeModel): JobModel | null;
    getParsedValue(value: any, node?: any): any;
}
