"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("@nocobase/utils");

  _utils = function _utils() {
    return data;
  };

  return data;
}

var _actions = require("./actions");

var _constants = require("../../constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// NOTE: for single record mode (mode: 0/null)
function middleware(_x, _x2) {
  return _middleware.apply(this, arguments);
}

function _middleware() {
  _middleware = _asyncToGenerator(function* (context, next) {
    const job = context.body,
          state = context.state,
          action = context.action;
    const _job$node$config = job.node.config,
          assignees = _job$node$config.assignees,
          mode = _job$node$config.mode; // NOTE: skip to no user implementation

    if (!assignees) {
      return next();
    }

    if (!state.currentUser) {
      return context.throw(401);
    }

    if (!assignees.includes(state.currentUser.id)) {
      return context.throw(404);
    } // NOTE: multiple record mode could not use jobs:submit action
    // should use users_jobs:submit/:id instead


    if (mode) {
      return context.throw(400);
    }

    yield next();
    const data = {
      userId: context.state.currentUser.id,
      jobId: job.id,
      nodeId: job.nodeId,
      executionId: job.executionId,
      workflowId: job.execution.workflowId,
      status: job.status,
      result: job.result
    }; // NOTE: update users job after main job is done

    const UserJobModel = context.db.getModel('users_jobs');
    let userJob = yield UserJobModel.findOne({
      where: {
        userId: context.state.currentUser.id,
        jobId: job.id
      }
    });

    if (userJob) {
      yield userJob.update(data);
    } else {
      userJob = yield UserJobModel.create(data);
    }
  });
  return _middleware.apply(this, arguments);
}

function run(_x3, _x4, _x5) {
  return _run.apply(this, arguments);
}

function _run() {
  _run = _asyncToGenerator(function* (node, prevJob, processor) {
    var _prevJob$id;

    const _node$config = node.config,
          assignees = _node$config.assignees,
          mode = _node$config.mode;

    if (!assignees) {
      const plugin = processor.options.plugin;
      const origin = plugin.instructions.get('prompt');
      return origin.constructor.prototype.run.call(this, node, prevJob, processor);
    }

    const job = yield processor.saveJob({
      status: _constants.JOB_STATUS.PENDING,
      result: mode ? [] : null,
      nodeId: node.id,
      upstreamId: (_prevJob$id = prevJob === null || prevJob === void 0 ? void 0 : prevJob.id) !== null && _prevJob$id !== void 0 ? _prevJob$id : null
    }); // NOTE: batch create users jobs

    const UserJobModel = processor.options.plugin.db.getModel('users_jobs');
    yield UserJobModel.bulkCreate(assignees.map(userId => ({
      userId,
      jobId: job.id,
      nodeId: node.id,
      executionId: job.executionId,
      workflowId: node.workflowId,
      status: _constants.JOB_STATUS.PENDING
    })), {
      transaction: processor.transaction
    });
    return job;
  });
  return _run.apply(this, arguments);
}

const PROMPT_ASSIGNED_MODE = {
  SINGLE: Symbol('single'),
  ALL: Symbol('all'),
  ANY: Symbol('any'),
  ALL_PERCENTAGE: Symbol('all percentage'),
  ANY_PERCENTAGE: Symbol('any percentage')
};
const Modes = {
  [PROMPT_ASSIGNED_MODE.SINGLE]: {
    getStatus(distribution, assignees) {
      const done = distribution.find(item => item.status !== _constants.JOB_STATUS.PENDING && item.count > 0);
      return done ? done.status : null;
    }

  },
  [PROMPT_ASSIGNED_MODE.ALL]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find(item => item.status === _constants.JOB_STATUS.RESOLVED);

      if (resolved && resolved.count === assignees.length) {
        return _constants.JOB_STATUS.RESOLVED;
      } // NOTE: `rejected` or `canceled`


      const failed = distribution.find(item => item.status < _constants.JOB_STATUS.PENDING);

      if (failed && failed.count) {
        return failed.status;
      }

      return null;
    }

  },
  [PROMPT_ASSIGNED_MODE.ANY]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find(item => item.status === _constants.JOB_STATUS.RESOLVED);

      if (resolved && resolved.count) {
        return _constants.JOB_STATUS.RESOLVED;
      }

      const failedCount = distribution.reduce((count, item) => item.status < _constants.JOB_STATUS.PENDING ? count + item.count : count, 0); // NOTE: all failures are considered as rejected for now

      if (failedCount === assignees.length) {
        return _constants.JOB_STATUS.REJECTED;
      }

      return null;
    }

  }
};

function getMode(mode) {
  switch (true) {
    case mode === 1:
      return Modes[PROMPT_ASSIGNED_MODE.ALL];

    case mode === -1:
      return Modes[PROMPT_ASSIGNED_MODE.ANY];

    case mode > 0:
      return Modes[PROMPT_ASSIGNED_MODE.ALL_PERCENTAGE];

    case mode < 0:
      return Modes[PROMPT_ASSIGNED_MODE.ANY_PERCENTAGE];

    default:
      return Modes[PROMPT_ASSIGNED_MODE.SINGLE];
  }
}

function resume(_x6, _x7, _x8) {
  return _resume.apply(this, arguments);
}

function _resume() {
  _resume = _asyncToGenerator(function* (node, job, processor) {
    var _job$latestUserJob$re, _job$latestUserJob, _getMode$getStatus;

    // NOTE: check all users jobs related if all done then continue as parallel
    const _node$config2 = node.config,
          assignees = _node$config2.assignees,
          mode = _node$config2.mode;

    if (!assignees) {
      const plugin = processor.options.plugin;
      const origin = plugin.instructions.get('prompt');
      return origin.constructor.prototype.resume.call(this, node, job, processor);
    }

    const UserJobModel = processor.options.plugin.db.getModel('users_jobs');
    const distribution = yield UserJobModel.count({
      where: {
        jobId: job.id
      },
      group: ['status']
    });
    const submitted = distribution.reduce((count, item) => item.status !== _constants.JOB_STATUS.PENDING ? count + item.count : count, 0);
    const result = mode ? (submitted || 0) / assignees.length : (_job$latestUserJob$re = (_job$latestUserJob = job.latestUserJob) === null || _job$latestUserJob === void 0 ? void 0 : _job$latestUserJob.result) !== null && _job$latestUserJob$re !== void 0 ? _job$latestUserJob$re : job.result;
    job.set({
      status: (_getMode$getStatus = getMode(mode).getStatus(distribution, assignees)) !== null && _getMode$getStatus !== void 0 ? _getMode$getStatus : _constants.JOB_STATUS.PENDING,
      result
    });
    return job;
  });
  return _resume.apply(this, arguments);
}

function _default(_x9) {
  return _ref.apply(this, arguments);
}

function _ref() {
  _ref = _asyncToGenerator(function* (plugin) {
    const instruction = plugin.instructions.get('prompt');
    instruction.extend({
      run,
      resume
    });
    instruction.use(middleware); // TODO(bug): through table should be load first because primary
    // await plugin.db.import({
    //   directory: path.join(__dirname, './collections')
    // });

    plugin.db.collection((0, _utils().requireModule)(_path().default.join(__dirname, './collections/users_jobs')));
    plugin.db.extendCollection((0, _utils().requireModule)(_path().default.join(__dirname, './collections/users')));
    plugin.db.extendCollection((0, _utils().requireModule)(_path().default.join(__dirname, './collections/jobs')));
    plugin.app.actions({
      'users_jobs:submit': _actions.submit
    });
  });
  return _ref.apply(this, arguments);
}