"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("../constants");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

class _default {
  constructor(plugin) {
    var _this = this;

    this.plugin = void 0;
    this.timers = new Map();

    this.run = /*#__PURE__*/function () {
      var _ref = _asyncToGenerator(function* (node, prevJob, processor) {
        var _prevJob$id;

        const job = yield processor.saveJob({
          status: _constants.JOB_STATUS.PENDING,
          result: null,
          nodeId: node.id,
          upstreamId: (_prevJob$id = prevJob === null || prevJob === void 0 ? void 0 : prevJob.id) !== null && _prevJob$id !== void 0 ? _prevJob$id : null
        });
        const duration = node.config.duration; // add to schedule

        _this.schedule(job, duration);

        return processor.end(node, job);
      });

      return function (_x, _x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }();

    this.resume = /*#__PURE__*/function () {
      var _ref2 = _asyncToGenerator(function* (node, prevJob, processor) {
        const endStatus = node.config.endStatus;
        prevJob.set('status', endStatus);
        return prevJob;
      });

      return function (_x4, _x5, _x6) {
        return _ref2.apply(this, arguments);
      };
    }();

    this.plugin = plugin;
    plugin.app.on('beforeStart', () => this.load());
    plugin.app.on('beforeStop', () => this.unload());
  }

  load() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const _this2$plugin$db$getC = _this2.plugin.db.getCollection('jobs'),
            model = _this2$plugin$db$getC.model;

      const jobs = yield model.findAll({
        where: {
          status: _constants.JOB_STATUS.PENDING
        },
        include: [{
          association: 'execution',
          attributes: [],
          where: {
            status: _constants.EXECUTION_STATUS.STARTED
          },
          required: true
        }, {
          association: 'node',
          attributes: ['config'],
          where: {
            type: 'delay'
          },
          required: true
        }]
      });
      jobs.forEach(job => {
        _this2.schedule(job, job.node.config.duration);
      });
    })();
  }

  unload() {
    var _iterator = _createForOfIteratorHelper(this.timers.values()),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        const timer = _step.value;
        clearTimeout(timer);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    this.timers = new Map();
  }

  schedule(job, duration) {
    const now = new Date();
    const createdAt = Date.parse(job.createdAt);
    const delay = createdAt + duration - now.getTime();
    const trigger = this.trigger.bind(this, job);
    this.timers.set(job.id, setTimeout(trigger, Math.max(0, delay)));
  }

  trigger(job) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const execution = yield job.getExecution();

      if (execution.status === _constants.EXECUTION_STATUS.STARTED) {
        const processor = _this3.plugin.createProcessor(execution);

        yield processor.resume(job);
      }

      if (_this3.timers.get(job.id)) {
        _this3.timers.delete(job.id);
      }
    })();
  }

}

exports.default = _default;